/*
 *                            COPYRIGHT
 *
 *  cschem - modular/flexible schematics editor - libcschem (core library)
 *  Copyright (C) 2019 Tibor 'Igor2' Palinkas
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.*
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/cschem
 *    lead developer: email to cschem (at) igor2.repo.hu
 *    mailing list: cschem (at) list.repo.hu (send "subscribe")
 */

#ifndef CSCH_CNC_ANY_OBJ_H
#define CSCH_CNC_ANY_OBJ_H

#include "concrete.h"

void csch_cobj_free(csch_chdr_t *obj);
csch_chdr_t *csch_cobj_dup(csch_sheet_t *sheet, csch_cgrp_t *grp, const csch_chdr_t *src, int keep_id);
void csch_cobj_update(csch_sheet_t *sheet, csch_chdr_t *src, int do_xform);
int csch_cobj_attrib_set(csch_sheet_t *sheet, csch_chdr_t *obj, int prio, const char *key, const char *val, const csch_source_arg_t source);
void csch_cobj_attrib_copy_all(csch_sheet_t *sheet, csch_chdr_t *dst, const csch_chdr_t *src);
int csch_cobj_attrib_seti(csch_sheet_t *sheet, csch_chdr_t *obj, int prio, const char *key, long idx, const char *val, const csch_source_arg_t source);


/* Fill in endpoint coords of object in x and y; side: 0 means query start,
   1 means end. Returns 0 on success. Works only on line and arc */
int csch_cobj_get_endxy(const csch_chdr_t *src, int side, csch_coord_t *x, csch_coord_t *y);

/* Select or unselect object; TODO: undoable? */
void csch_cobj_select(csch_sheet_t *sheet, csch_chdr_t *obj);
void csch_cobj_unselect(csch_sheet_t *sheet, csch_chdr_t *obj);

/* Unselet object and all parent objects */
csch_inline void csch_chdr_unselect_wp(csch_chdr_t *obj)
{
	csch_sheet_t *sheet = obj->sheet;
	for(;obj != NULL; obj = &obj->parent->hdr)
		csch_cobj_unselect(sheet, obj);
}


/* Modify obj's pen name(s) if not NULL; if unodable, add the operation to the
   undo list */
void csch_chdr_pen_name_modify(csch_sheet_t *sheet, csch_chdr_t *obj, csch_comm_str_t *stroke_name, csch_comm_str_t *fill_name, int undoable);

#endif
