/*
 *                            COPYRIGHT
 *
 *  cschem - modular/flexible schematics editor - libcschem (core library)
 *  Copyright (C) 2018 Tibor 'Igor2' Palinkas
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.*
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/cschem
 *    lead developer: email to cschem (at) igor2.repo.hu
 *    mailing list: cschem (at) list.repo.hu (send "subscribe")
 */


#ifndef CSCH_MESSAGE_H
#define CSCH_MESSAGE_H

#include <stdarg.h>

typedef enum csch_message_level_e {
	CSCH_MSG_DEBUG = 0,   /* Debug message. Should probably not be shown in regular operation. */
	CSCH_MSG_INFO,        /* Info message. FYI for the user, no action needed. */
	CSCH_MSG_WARNING,     /* Something the user should probably take note */
	CSCH_MSG_ERROR        /* Couldn't finish an action, needs user attention. */
} csch_message_level_t;

/* printf-like logger to the log dialog and stderr */
void csch_message(csch_message_level_t level, const char *format, ...);

/* error-macro-friendly shorthand for csch_messgae(CSCH_MSG_ERROR,) */
void csch_error(const char *format, ...);

/* User provided message handlers; return zero if the message could not be printed/handled */
typedef int (*csch_message_handler_t)(csch_message_level_t level, const char *format, va_list ap);

/* (un)register handler; handlers are called in arbitrary order */
void csch_message_reg_handler(csch_message_handler_t handler);
void csch_message_unreg_handler(csch_message_handler_t handler);

#endif
