/*
 *                            COPYRIGHT
 *
 *  cschem - modular/flexible schematics editor - libcschem (core library)
 *  Copyright (C) 2018,2019 Tibor 'Igor2' Palinkas
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.*
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/cschem
 *    lead developer: email to cschem (at) igor2.repo.hu
 *    mailing list: cschem (at) list.repo.hu (send "subscribe")
 */


#ifndef CSCH_PROJECT_H
#define CSCH_PROJECT_H

#include <libcschem/common_types.h>
#include <libcschem/concrete.h>
#include <libcschem/attrib.h>
#include <libcschem/engine.h>
#include <genvector/vtp0.h>
#include <genvector/vts0.h>
#include <libfungw/fungw.h>

struct csch_view_s {
	fgw_ctx_t fgw_ctx; /* name of the view is the name of the context */
	vtp0_t engines; /* priority-ordered list of (csch_view_eng_t *); highest prio (lowest numer) is at the end */
	csch_project_t *parent;
};

struct csch_project_s {
	char *loadname; /* file name as specified by the user */
	char *filename; /* real file name */
	char *prjdir;   /* real path to the directory that hosts the project file */
	vtp0_t sheets;
	vts0_t symlibs;
	vtp0_t views;   /* of (csch_view_t *) */
	int curr;       /* index of currently active view */
};

csch_project_t *csch_project_alloc(void);
void csch_project_free(csch_project_t *prj);

/* Create a new view, returns the index; returns -1 on error */
csch_view_t *csch_view_create(csch_project_t *prj, const char *name);

/* free all fields and view itself (but do not remove from parent!) */
void csch_view_free(csch_view_t *view);


/* Create a new engine in a view, reassign priorities of other engines in
   the view as needed. On success return the index of the new engine (from 0);
   on failure return -1 */
int csch_view_eng_append(csch_view_t *view, const char *user_name, const char *eng_name, const char *file_name);
int csch_view_eng_insert_before(csch_view_t *view, const char *user_name, const char *eng_name, const char *file_name, int idx);


/* Recalculate the ->filename of prj using its ->loadname field */
int csch_proj_update_filename(csch_project_t *prj);

/* recalculate the filename (real, full path) of sheet, using prj's filename
   for CWD in case of relative path */
int csch_proj_sheet_update_filename(csch_project_t *prj, csch_sheet_t *sheet);

/*** loaders ***/
int csch_project_load_sheet(csch_project_t *prj, const char *fn, const char *fmt, csch_sheet_t **sheet_out);
csch_project_t *csch_load_project_by_sheet_name(const char *sheet_fn);


#endif
